import { VerificationError, verify_number, verify_string, verify_boolean, verify_void, strip_number, strip_string, strip_boolean, strip_void } from "./ts_base";
import TestAtom, { verify_TestAtom, strip_TestAtom } from "./TestAtom";
import TestEnum, { verify_TestEnum, strip_TestEnum } from "./TestEnum";
export default class Test {
   atom?: TestAtom;
   array?: TestAtom[];
   enumValue?: TestEnum;
   map?: { [key: number]: TestAtom };

   constructor(init?:Partial<Test>){
      if(init){
         if(init["atom"])
            this.atom = init["atom"];
         if(init["array"])
            this.array = init["array"];
         if(init["enumValue"])
            this.enumValue = init["enumValue"];
         if(init["map"])
            this.map = init["map"];
      }
   }


   static verify(data: Test) {
      verify_Test(data);
   }
}

export function strip_Test(data: Test): Test {
   let res = {} as any;
   res["atom"] = strip_TestAtom(data["atom"])
   res["array"] = data["array"].map(elm=>
      strip_TestAtom(elm)
   )
   res["enumValue"] = strip_TestEnum(data["enumValue"])
   res["map"] = {}
   Object.entries(data["map"]).forEach(([key, val]) => res["map"][key] = strip_TestAtom(val))
   return res;
}

export function verify_Test(data: Test) {
   if(data["atom"] !== null && data["atom"] !== undefined ) {
      if(!verify_TestAtom(data["atom"])) throw new VerificationError("TestAtom", "atom", data["atom"]);
   }

   if(data["array"] !== null && data["array"] !== undefined ) {
      if(!Array.isArray(data["array"])) throw new VerificationError("array", "array", data["array"]);
      for(const elm of data["array"]) {
         if(!verify_TestAtom(elm)) throw new VerificationError("TestAtom", "array", elm);
      }
   }

   if(data["enumValue"] !== null && data["enumValue"] !== undefined ) {
      if(!verify_TestEnum(data["enumValue"])) throw new VerificationError("TestEnum", "enumValue", data["enumValue"]);
   }

   if(data["map"] !== null && data["map"] !== undefined ) {
      if(typeof data["map"] !== "object") throw new VerificationError("object", map, data["map"]);
      for(const key in data["map"]) {
         if(!verify_TestAtom(data["map"][key])) throw new VerificationError("TestAtom", "map", data["map"][key]);
      }
   }

   return data
}