import { VerificationError, verify_number, verify_string, verify_boolean, verify_void, strip_number, strip_string, strip_boolean, strip_void } from "./ts_base";
import AddValueResponse, { verify_AddValueResponse, strip_AddValueResponse } from "./AddValueResponse";
import AddValueRequest, { verify_AddValueRequest, strip_AddValueRequest } from "./AddValueRequest";
export type {
   AddValueResponse,
   AddValueRequest,
}
import { Service } from "./service_server";

export abstract class TestService<T> extends Service<T> {
   public name = "TestService";
   constructor(){
      super();
      this.functions.add("AddValuesSingleParam")
      this.functions.add("AddValuesMultipleParams")
      this.functions.add("ReturningVoid")
      this.functions.add("OnEvent")
      this.functions.add("FunctionWithArrayAsParamAndReturn")
   }

   abstract AddValuesSingleParam(request: AddValueRequest, ctx: T): Promise<AddValueResponse>;
   _AddValuesSingleParam(params: any[] | any, ctx: T): Promise<AddValueResponse> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
      } else {
         p.push(params["request"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      verify_AddValueRequest(p[0])
      }
      
      p.push(ctx);
      return this.AddValuesSingleParam.call(this, ...p).then(res => verify_AddValueResponse(strip_AddValueResponse(res)));
   }

   abstract AddValuesMultipleParams(value1: number, value2: number, ctx: T): Promise<number>;
   _AddValuesMultipleParams(params: any[] | any, ctx: T): Promise<number> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
      } else {
         p.push(params["value1"])
         p.push(params["value2"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      verify_number(p[0])
      }
      if(p[1] !== null && p[1] !== undefined) {
      verify_number(p[1])
      }
      
      p.push(ctx);
      return this.AddValuesMultipleParams.call(this, ...p).then(res => verify_number(strip_number(res)));
   }

   abstract ReturningVoid(param1: number, ctx: T): Promise<void>;
   _ReturningVoid(params: any[] | any, ctx: T): Promise<void> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
      } else {
         p.push(params["param1"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      verify_number(p[0])
      }
      
      p.push(ctx);
      return this.ReturningVoid.call(this, ...p).then(res => verify_void(strip_void(res)));
   }

   abstract OnEvent(param1: string, ctx: T): void;
   _OnEvent(params: any[] | any, ctx: T): void {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
      } else {
         p.push(params["param1"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      verify_string(p[0])
      }
      
      p.push(ctx);
      return this.OnEvent.call(this, ...p)
   }

   abstract FunctionWithArrayAsParamAndReturn(values1: number[], values2: number[], ctx: T): Promise<number[]>;
   _FunctionWithArrayAsParamAndReturn(params: any[] | any, ctx: T): Promise<number[]> {
      let p: any[] = [];
      if(Array.isArray(params)){
         p = params;
      } else {
         p.push(params["values1"])
         p.push(params["values2"])
      }
      
      if(p[0] !== null && p[0] !== undefined) {
      for(const elm of p[0]) {
         verify_number(elm)
      }
      }
      if(p[1] !== null && p[1] !== undefined) {
      for(const elm of p[1]) {
         verify_number(elm)
      }
      }
      
      p.push(ctx);
      return this.FunctionWithArrayAsParamAndReturn.call(this, ...p).then(res => res.map(e => verify_number(strip_number(e))));
   }

}